import { CompoundedComponent } from "@/types";
import { register } from "@antv/x6-react-shape";
import { Graph, Node } from "@antv/x6";
import { ports, defaultData } from "../data";
import CustomInput from "../CustomInput";
import { useSizeHook, useShapeProps } from "@/hooks";
const component = ({ node }: { node: Node }) => {
  const { label, text, fill, stroke, opacity } = node.getData();
  const { size, ref } = useSizeHook();
  const {
    fillContent,
    defsContent,
    strokeColor,
    strokeWidth,
    strokeDasharray,
  } = useShapeProps(fill, size, stroke);
  // 计算正五边形的顶点坐标
  const getPoints = (width: number, height: number) => {
    const cx = width / 2;
    const cy = height / 2;
    const outerRadius = Math.min(cx, cy) * 0.8; // 外层半径
    const innerRadius = outerRadius * 0.382; // 内层半径（黄金比例）

    let points = "";
    // 外层顶点
    for (let i = 0; i < 5; i++) {
      const angle1 = 72 * i * (Math.PI / 180); // 每个顶点的角度
      points += `${cx + outerRadius * Math.cos(angle1)},${cy + outerRadius * Math.sin(angle1)} `;

      const angle2 = (72 * i + 36) * (Math.PI / 180); // 每个顶点的角度
      points += `${cx + innerRadius * Math.cos(angle2)},${cy + innerRadius * Math.sin(angle2)} `;
    }

    return points;
  };

  return (
    <>
      <div
        className="relative text-0 w-full h-full"
        ref={ref}
        style={{ opacity: opacity / 100 }}
      >
        <CustomInput value={label} styles={text} node={node} />
        <svg
          className="w-full h-full"
          viewBox={`0 0 ${size?.width} ${size?.height}`}
          xmlns="http://www.w3.org/2000/svg"
        >
          <defs>{defsContent}</defs>
          <polygon
            points={getPoints(size?.width, size?.height)}
            fill={fillContent}
            stroke={strokeColor}
            strokeDasharray={strokeDasharray}
            strokeWidth={strokeWidth}
          />
        </svg>
      </div>
    </>
  );
};

register({
  shape: "custom-react-fiveStar",
  width: 90,
  height: 90,
  effect: ["data"],
  component: component,
});

// 左侧连接桩
Graph.registerPortLayout("fiveStarLeft", (portsPositionArgs, elemBBox) => {
  return portsPositionArgs.map((item) => {
    return {
      ...item,
      position: {
        x: 0,
        y: elemBBox.height * 0.4,
      },
    };
  });
});

// 右侧连接桩
Graph.registerPortLayout("fiveStarRight", (portsPositionArgs, elemBBox) => {
  return portsPositionArgs.map((item) => {
    return {
      ...item,
      position: {
        x: elemBBox.width,
        y: elemBBox.height * 0.4,
      },
    };
  });
});

const FiveStar: CompoundedComponent = {
  name: "五角星",
  icon: require("./image/pentagon.png"),
  node: {
    shape: "custom-react-fiveStar",
    data: {
      label: "五角星",
      ...defaultData,
    },
    ports: {
      ...ports,
      groups: {
        ...ports.groups,
        fiveStarLeft: {
          position: "fiveStarLeft",
          attrs: {
            circle: {
              r: 4,
              magnet: true,
              stroke: "#5F95FF",
              strokeWidth: 1,
              fill: "#fff",
              style: {
                visibility: "hidden",
              },
            },
          },
        },
        fiveStarRight: {
          position: "fiveStarRight",
          attrs: {
            circle: {
              r: 4,
              magnet: true,
              stroke: "#5F95FF",
              strokeWidth: 1,
              fill: "#fff",
              style: {
                visibility: "hidden",
              },
            },
          },
        },
      },
      items: [
        {
          group: "top",
          args: {
            dy: 2,
          },
        },
        {
          group: "fiveStarLeft",
        },
        {
          group: "bottom",
          args: {
            dy: -2,
          },
        },
        {
          group: "fiveStarRight",
        },
      ],
    },
  },
};

export default FiveStar;
