import { useEffect, useRef } from "react";
import { Flex, Tabs } from "antd";
import styles from "./index.less";
import "./index.less";
import { Graph, Shape } from "@antv/x6";
import { Scroller } from "@antv/x6-plugin-scroller";
import Libary from "../Libary";
import { useModel } from "umi";
import { LineType } from '@/enum';
import { defaultData } from '@/components/data';
export default function Content() {
  const stageRef = useRef<HTMLDivElement | null>(null);
  const { initGraph } = useModel("graphModel");
  const { onChangePageSettings } = useModel("appModel");

  useEffect(() => {
    if(!stageRef.current) return;

    const { width, height } = stageRef.current?.getBoundingClientRect();
    onChangePageSettings('width', parseInt(width + ''));
    onChangePageSettings('height', parseInt(height + ''));
  }, [stageRef.current]);

  useEffect(() => {
    const graph = new Graph({
      container: document.getElementById("graph-container")!,
      autoResize: true,
      width: 2000,
      height: 2000,
      mousewheel: {
        enabled: true,
        zoomAtMousePosition: true,
        modifiers: "ctrl",
        minScale: 0.2,
        maxScale: 2,
      },
      // 查找父节点
      embedding: {
        enabled: true,
        findParent({ node }) {
          const bbox = node.getBBox();
          return this.getNodes().filter((node) => {
            const data = node.getData<{ isPage: boolean }>();
            if (data && data.isPage) {
              const targetBBox = node.getBBox();
              return bbox.isIntersectWithRect(targetBBox);
            }
            return false;
          });
        },
      },
      background: {
        color: "#eaecee",
      },
      interacting: {
        edgeLabelMovable: false,
        edgeMovable: true,
        nodeMovable: (view) => {
          const data = view.cell.getData<{ isPage: boolean }>();
          return !data || !data.isPage;
        },
        arrowheadMovable: true,
        vertexMovable: true,
        vertexAddable: true,
        vertexDeletable: true,
        useEdgeTools: true,
        magnetConnectable: true,
        stopDelegateOnDragging: true,
        toolsAddable: true,
      },
      // 连接配置
      connecting: {
        allowEdge: true,
        allowLoop: true,
        allowNode: true,
        allowBlank: true,
        allowPort: true,
        allowMulti: true,
        highlight: false,
        anchor: "center",
        connectionPoint: "anchor",
        snap: {
          radius: 20,
        },
        createEdge() {
          return new Shape.Edge({
            router: "manhattan",
            attrs: {
              line: {
                stroke: "#323232",
                strokeDasharray: LineType.solid,
                strokeWidth: 2,
                sourceMarker: {
                  name: ''
                },
                targetMarker: {
                  name: 'block',
                },
              },
              text: defaultData.text
            },
          });
        },
        validateConnection({ targetMagnet }) {
          return !!targetMagnet;
        },
      },
      // 高亮显示
      highlighting: {
        magnetAdsorbed: {
          name: "stroke",
          args: {
            attrs: {
              fill: "#5F95FF",
              stroke: "#5F95FF",
            },
          },
        },
      },
    });

    graph.use(
      new Scroller({
        enabled: true,
      })
    );
    initGraph(graph);
  }, []);

  return (
    <Flex className="h-full">
      <div className={styles.leftPanel}>
        <Tabs
          className="h-full"
          centered
          items={[
            {
              key: "1",
              label: "组件库",
              children: <Libary />,
            },
            {
              key: "2",
              label: "风格",
              children: <div>风格面板</div>,
            },
          ]}
        />
      </div>
      <div
        className={styles.designLayout}
        ref={stageRef}
        style={{ width: "100%", height: "100%" }}
      >
        <div id="graph-container"></div>
      </div>
    </Flex>
  );
}
